/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
* DUAL SCALE DISPLAY                                                              Version 1.1  2/24/2022 *
* Created by: W. van Leeuwen (Billv923@outlook.com)                                                      *
*                                                                                                        *
* This sketch provides a graphical interface that works in conjunction with the author's Arduino UNO     *
* microcontroller based pinewood derby car scale which measures the weight of both the front and rear    *
* axels and calculates the center-of-gravity (CG) for use in placing weights on the car.                 *
*                                                                                                        *
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
   
  import processing.serial.*;                        //Use the serial library
  import static javax.swing.JOptionPane.*;           //Use the option box dialog library

//Variables
  int PageSelect = 1;                                //1=Intro page, 2=Weight page
  int i, j, x, y;                                    //Working integer variables
  int Rx = 250;                                      //Intro page Background color Red value
  int Gx = 250;                                      //Intro page Background color Green value
  int Bx = 150;                                      //Intro page Background color Blue value
  int Ry = 140;                                      //Weight page Background color Red value
  int Gy = 200;                                      //Weight page Background color Green value
  int By = 230;                                      //Weight page Background color Blue value
  int Axle1_Pos, Axle2_Pos;                          //Front & Rear Axle position in pixels (1 inch = 100 pixels)
  int CG_Pos, ICG_Pos;                               //CG & Ideal CG pixel position
  boolean ComPort_Avail;                             //Denotes when com port is available
  boolean IntroPg_Active;                            //Set to true when Intro page is displayed
  float Scale1_Wt, Scale2_Wt, Total_Wt;              //Variables for scale weights in ounces
  float Wheelbase, CG, Ideal_CG;                     //Variables for wheelbase and CG lengths in inches
  String Str1 = "";                                  //Working string variable
  String prompt = "";                                //Variable used for storing popup message text
  String inString = "";                              //Input string from serial port
  String COMx, COMlist = "";                         //COM Port check/select routine variables
  String Portname = "";                              //COM Port name variable
  PFont font;                                        //Set font
  PImage logo;                                       //Define an image variable as 'logo'
  PImage car1;                                       //Define an image variable as 'car1'
  PShape cgArrow;                                    //Create PShape object called "cgArrow"
  Serial myPort;                                     //Set the Serial port name to myPort
  
//*********************************************************************************************************
void setup() {
  size(1200, 700);                                   //Set display area in pixels
  logo = loadImage("BillV_Logo.png");                //Load "Bill V's Derby Timers" Logo
  car1 = loadImage("Car_On_Scale.png");              //Load image of car on the scale
  
  //Create a custom up-arrow shape for designating the CG location
  cgArrow = createShape();                           //Create cgArrow object
  cgArrow.beginShape();                              //Begin shape creation
  cgArrow.fill(0, 0, 255);                           //Set shape color
  cgArrow.vertex(25, 0);                             //Define vertex point
  cgArrow.vertex(0, 50);                             //Define vertex point
  cgArrow.vertex(15, 50);                            //Define vertex point
  cgArrow.vertex(15, 120);                           //Define vertex point
  cgArrow.vertex(35, 120);                           //Define vertex point
  cgArrow.vertex(35, 50);                            //Define vertex point
  cgArrow.vertex(50, 50);                            //Define vertex point
  cgArrow.vertex(25, 0);                             //Define vertex point
  cgArrow.endShape(CLOSE);                           //End shape
  
  //Set the font. The font must be located in the sketch's "data" directory to load successfully
  font = loadFont("ArialNarrow-Bold-48.vlw");
  textFont(font, 60);
  PageSelect = 1;                                    //Set selection to Setup page
  IntroPg_Active = false;                            //Set if Intro Page is active
  
  //Establish Communications with scale
  Select_Comm();                                     //Call subroutine to select com port
  
  Wheelbase = 4.375;                                 //Set default wheelbase
  CG = 0.75;                                         //Set default CG
  Ideal_CG = 0.75;                                   //Set default ideal CG

} //End of "Setup" routine

//*********************************************************************************************************
void draw() {  //Note: Things aren't displayed on the screen until the draw loop completes.
  //if (ComPort_Avail && inString.indexOf("WT") >= 0) { //Check for receipt of 'Weight' message
  if (inString.indexOf("WT") >= 0) { //Check for receipt of 'Weight' message
    //Note: Weight message has following format: "WT: xx.xxx yy.yyy z.zzz"
    //  where: xx.xxx = Scale1 weight, yy.yyy = Scale2 weight & z.zzz = CG
    Str1 = inString.substring(4, 10);                //Extract Scale1 weight value
    Scale1_Wt = float(Str1) - 10.0;                  //and convert it to a floating point number
    Str1 = inString.substring(11, 17);               //Extract Scale2 weight value
    Scale2_Wt = float(Str1) -10.0;                   //and convert it to a floating point number
    Str1 = inString.substring(18, 23);               //Extract CG value
    CG = float (Str1);                               //and convert it to a floating point number
    Total_Wt = Scale1_Wt + Scale2_Wt;                //Calculate total weight
    
    if (Total_Wt > 0.1) {                            //Is there a car on the scale?
      Draw_Weight_Page();                            //Call subroutine to draw the Weight page
      IntroPg_Active = false;                        //Reset to allow Intro page to be drawn if car is removed
    }
    else {                                           //No...Display the Intro Page if no car on scale
      if (!IntroPg_Active) {                         //Do if Intro page is not active
        Draw_Intro_Page();                           //Call subroutine to draw the Intro page
        IntroPg_Active = true;                       //Set so we don't draw it again
        Display_Wheelbase();                         //Call subroutine to display the wheelbase
      }
    }
  }
  if (ComPort_Avail && inString.indexOf("WB") >= 0) { //Check for receipt of 'Wheelbase' message
    //Note: Wheelbase message has following format: "WB: x.xxx"  where: x.xxx = Wheelbase
    Str1 = inString.substring(4, 9);                 //Extract wheelbase value
    Wheelbase = float (Str1);                        //and convert it to a floating point number
    Display_Wheelbase();                             //Call subroutine to display the wheelbase
  }
  
} //End of "draw" loop

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
*                                                SUBROUTINES                                               *
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
void serialEvent(Serial p) {                         //Serial data handling routine
  inString = p.readString();
}

void Draw_Intro_Page() {                             //Subroutine - Draw the Intro page
  background(Rx, Gx, Bx);                            //Clear screen with background color
  textAlign(CENTER);                                 //Set text alignment to center justify
  textSize(90);                                      //Set text size to 90 pixels
  fill(255, 0, 170);                                 //Set Title color
  text("WEIGH YOUR CAR",600,80);                     //Display title line 1
  textSize(70);                                      //Set text size to 80 pixels
  text("&",600,160);                                 //Display title line 2
  text("CHECK YOUR CENTER-OF-GRAVITY (CG)",600,240); //Display title line 3
  text("HERE",600,320);                              //Display title line 4
  textAlign(LEFT);                                   //Set text alignment to left justify
  textSize(40);                                      //Set text size to 25 pixels
  fill(0);                                           //Set color to black
  text("To start:",15,400);
  text("Step 1: Measure your car's wheelbase (Distance between front & rear axle).",25,450);
  text("Step 2: Use the + and - buttons to adjust the scale's wheelbase setting.",25,500);
  text("Step 3: Place your car on the scale as shown below:",25,550);
  image(car1, 650, 570);                             //Display car1 image
  image(logo, 1080, 640);                            //Display logo it in lower right corner
}

void Draw_Weight_Page() {                            //Subroutine - Draw the Weight page
  background(Ry, Gy, By);                            //Clear screen with Weight page background color
  textAlign(CENTER);                                 //Set text alignment to center
  textSize(90);                                      //Set text size to 80 pixels
  fill(255, 0, 170);                                 //Set Title color
  text("WEIGHT & CG",600,100);                       //Display title line 1
  textSize(70);                                      //Set text size to 80 pixels
  fill(204, 102, 0);                                 //Set color to burnt orange
  quad(250, 150, 250, 275, 950, 275, 950, 250);      //Draw car body
  Axle1_Pos = 600 - int(Wheelbase/2.0 * 100);        //Determine rear axle graphic position in pixels
  Axle2_Pos = 600 + int(Wheelbase/2.0 * 100);        //Determine front axle graphic position in pixels
  Ideal_CG = 0.1714 * Wheelbase;                     //Calculate ideal CG in inches
  ICG_Pos = int(Ideal_CG * 100) +  Axle1_Pos;        //Determine ideal CG graphic position in pixels
  CG_Pos = int(CG * 100) +  Axle1_Pos;               //Determine actual CG graphic position in pixels
  shape(cgArrow, CG_Pos-25, 300);                    //Draw cgArrow
  Draw_CGbar(ICG_Pos-37, 280);                       //Call subroutine to draw ideal CG bar
  fill(0, 0, 0, 75);                                 //Set color to black with transparancy
  circle(Axle1_Pos, 265, 118.75);                    //Draw rear wheel at rear axle position
  circle(Axle2_Pos, 265, 118.75);                    //Draw front wheel at front axle position
  fill(255);                                         //Set color to white
  circle(Axle1_Pos, 265, 20.0);                      //Draw rear wheel center axle
  circle(Axle2_Pos, 265, 20.0);                      //Draw front wheel center axle
  Draw_Pointer(ICG_Pos+45, 300);                     //Draw pointer to the CG bar
  fill(0);                                           //Set color to black
  textSize(40);                                      //Set text size to 40 pixels
  textAlign(CENTER);                                 //Set text alignment to center justify
  text("Your Car's CG", CG_Pos, 450);                //Draw text
  text("Position", CG_Pos, 490);                     //Draw text
  textAlign(LEFT);                                   //Set text alignment to left justify
  text("Ideal CG Position", 875, 400);               //Draw text
  text("Rear Axle Weight:", 350, 575);               //Draw text
  text(Scale1_Wt, 650, 575);                         //Draw rear axle weight
  text("oz", 750, 575);                              //Draw weight suffix
  text("Front Axle Weight:", 350, 625);              //Draw text
  text(Scale2_Wt, 650, 625);                         //Draw rear axle weight
  text("oz", 750, 625);                              //Draw weight suffix
  textSize(50);                                      //Set text size to 50 pixels
  fill(255, 0, 0);                                   //Set color to red
  text("Total Weight:", 350, 675);                   //Draw text
  text(Total_Wt, 645, 675);                          //Draw rear axle weight
  text("oz", 770, 675);                              //Draw weight suffix
  image(logo, 1080, 640);                            //Display logo it in lower right corner
}

void Draw_CGbar(int x, int y) {                      //Subroutine - Draw CG color bar
  fill(240, 255, 0);                                 //Set color to yellow
  rect(x, y, 25, 20);                                //Draw left rectangle
  rect(x+50, y, 25, 20);                             //Draw right rectangle
  fill(0, 255, 0);                                   //Set color to green
  rect(x+25, y, 25, 20);                             //Draw right rectangle
}

void Draw_Pointer(int x, int y) {                    //Subroutine - Draw pointer
  strokeWeight(2);                                   //Set line thickness to 2 pixels
  line(x, y, 870, 390);                              //Draw lines
  line(x, y, x+5, y+10);
  line(x, y, x+10, y-5);
  strokeWeight(1);                                   //Reset line thickness to default
}

void Display_Wheelbase() {                           //Subroutine - Display the wheelbase
  if (IntroPg_Active) {                              //Do only if Intro page is active
    fill(255);                                       //Set color to white
    rect(150, 570, 300, 100, 10);                    //Draw display box rectangle
    fill(0);                                         //Set color to black
    textSize(25);                                    //Set text size to 40 pixels
    textAlign(CENTER);                               //Set text alignment to center justify
    text("Current Wheelbase", 300, 600);             //Draw text
    text("Setting", 300, 630);                       //Draw text
    text(Wheelbase, 265, 660);                       //Draw wheelbase value
    text("inches", 335, 660);                        //Draw text
  }
}

void Select_Comm() {
  ComPort_Avail = false;                             //Default COM port available to false
    //Check & Open the serial port at 9600 baud and buffer data till linefeed received
  try {                                              //Use the 'try' & 'catch' exception handling process
    i = Serial.list().length;                        //Get the number of open ports
    if (i != 0) {                                    //Do if at least one port is open
      if (i >= 2) {                                  //Do if more than one open port
        //Create a list of found COM ports from which the user can pick
        for (j = 0; j < i; j++) {
          COMlist += char(j+'a') + " = " + Serial.list()[j];
          if (j+1 < i) COMlist += ",  ";
        }
        COMx = showInputDialog("Which COM port is correct? (a,b,..):\n"+COMlist);
        if (COMx == null) ComPort_Avail = false;;
        if (COMx.isEmpty()) ComPort_Avail = false;;
        i = int(COMx.toLowerCase().charAt(0) - 'a') + 1;
      }
      Portname = Serial.list()[i-1];                 //Get the selected port name
      myPort = new Serial(this, Portname, 115200);   //Activate COM port at 115200 baud
      myPort.bufferUntil('\n');                      //Buffer until Line Feed appears
      prompt = "COM Port Selected: " + Portname;     //Construct COM message & COM Port
      showMessageDialog(null,prompt);                //Display the prompt for acknowledgement
      ComPort_Avail = true;                          //Set Com Port avail flag
    }
    else {
      showMessageDialog(null,"No open/connected COM ports detected.\nFix problem & try again");
    }
  }
  catch (Exception e) { //Error catch routine. Print the type of error
    showMessageDialog(null,"COM port is not available\n(may be in use by another program)");
    //println("Error:", e);                          //For debug only
    //exit();                                        //For debug only
  }
}
